//
// This mirrors UKeymanTargets.pas
//
export var KeymanTarget;
(function (KeymanTarget) {
    KeymanTarget["any"] = "any";
    KeymanTarget["windows"] = "windows";
    KeymanTarget["macosx"] = "macosx";
    KeymanTarget["linux"] = "linux";
    KeymanTarget["web"] = "web";
    KeymanTarget["iphone"] = "iphone";
    KeymanTarget["ipad"] = "ipad";
    KeymanTarget["androidphone"] = "androidphone";
    KeymanTarget["androidtablet"] = "androidtablet";
    KeymanTarget["mobile"] = "mobile";
    KeymanTarget["desktop"] = "desktop";
    KeymanTarget["tablet"] = "tablet";
})(KeymanTarget || (KeymanTarget = {}));
;
export const 
// Note: if these change, update the copied values in KeyboardParser.pas accordingly
AllKeymanTargets = [
    KeymanTarget.any,
    KeymanTarget.windows, KeymanTarget.macosx, KeymanTarget.linux,
    KeymanTarget.web, KeymanTarget.iphone, KeymanTarget.ipad, KeymanTarget.androidphone, KeymanTarget.androidtablet,
    KeymanTarget.mobile, KeymanTarget.desktop, KeymanTarget.tablet
], TouchKeymanTargets = [
    KeymanTarget.iphone, KeymanTarget.ipad, KeymanTarget.androidphone, KeymanTarget.androidtablet,
    KeymanTarget.mobile, KeymanTarget.tablet
], 
// Compile to .kmx
KMXKeymanTargets = [
    KeymanTarget.windows, KeymanTarget.macosx, KeymanTarget.linux, KeymanTarget.desktop
], 
// Compile to .js
KMWKeymanTargets = [
    KeymanTarget.web, KeymanTarget.iphone, KeymanTarget.ipad, KeymanTarget.androidphone, KeymanTarget.androidtablet,
    KeymanTarget.mobile, KeymanTarget.tablet
], 
// Supports .kvks
KeymanTargetsUsingKVK = [
    KeymanTarget.windows, KeymanTarget.macosx, KeymanTarget.linux, KeymanTarget.desktop, KeymanTarget.web
], 
// Friendly names for targets, e.g. for documentation
SKeymanTargetNames = {
    [KeymanTarget.any]: 'All',
    [KeymanTarget.windows]: 'Windows', [KeymanTarget.macosx]: 'macOS', [KeymanTarget.linux]: 'Linux',
    [KeymanTarget.web]: 'Web', [KeymanTarget.iphone]: 'iPhone', [KeymanTarget.ipad]: 'iPad',
    [KeymanTarget.androidphone]: 'Android phone', [KeymanTarget.androidtablet]: 'Android tablet',
    [KeymanTarget.mobile]: 'Mobile devices', [KeymanTarget.desktop]: 'Desktop devices',
    [KeymanTarget.tablet]: 'Tablet devices'
};
export function keymanTargetsFromString(targets, options) {
    let result = new Set(targets.split(/ +/));
    if (options?.expandTargets) {
        if (result.has(KeymanTarget.any)) {
            return AllKeymanTargets;
        }
        if (result.has(KeymanTarget.mobile)) {
            result.delete(KeymanTarget.mobile);
            result.add(KeymanTarget.androidphone);
            result.add(KeymanTarget.iphone);
        }
        if (result.has(KeymanTarget.tablet)) {
            result.delete(KeymanTarget.tablet);
            result.add(KeymanTarget.androidtablet);
            result.add(KeymanTarget.ipad);
        }
        if (result.has(KeymanTarget.desktop)) {
            result.delete(KeymanTarget.desktop);
            result.add(KeymanTarget.linux);
            result.add(KeymanTarget.macosx);
            result.add(KeymanTarget.windows);
        }
    }
    return [...result];
}
//# sourceMappingURL=keyman-targets.js.map