declare const _default: {
    $schema: string;
    $ref: string;
    definitions: {
        package: {
            type: string;
            properties: {
                system: {
                    $ref: string;
                };
                options: {
                    $ref: string;
                };
                startMenu: {
                    $ref: string;
                };
                strings: {
                    $ref: string;
                };
                files: {
                    $ref: string;
                };
                keyboards: {
                    $ref: string;
                };
                lexicalModels: {
                    $ref: string;
                };
                info: {
                    $ref: string;
                };
                relatedPackages: {
                    type: string;
                    items: {
                        $ref: string;
                    };
                };
            };
            additionalProperties: boolean;
            required: string[];
        };
        system: {
            type: string;
            properties: {
                keymanDeveloperVersion: {
                    type: string;
                };
                fileVersion: {
                    type: string;
                };
            };
            additionalProperties: boolean;
            required: string[];
        };
        options: {
            type: string;
            properties: {
                readmeFile: {
                    type: string;
                };
                graphicFile: {
                    type: string;
                };
                licenseFile: {
                    type: string;
                };
                welcomeFile: {
                    type: string;
                };
                executeProgram: {
                    type: string;
                };
                msiFilename: {
                    type: string;
                };
                msiOptions: {
                    type: string;
                };
            };
            additionalProperties: boolean;
        };
        startMenu: {
            type: string;
            properties: {
                folder: {
                    type: string;
                };
                addUninstallEntry: {
                    type: string;
                };
                items: {
                    type: string;
                    items: {
                        $ref: string;
                    };
                };
            };
            additionalProperties: boolean;
        };
        startMenuItem: {
            type: string;
            properties: {
                name: {
                    type: string;
                };
                filename: {
                    type: string;
                };
                arguments: {
                    type: string;
                };
                icon: {
                    type: string;
                };
                location: {
                    type: string;
                };
            };
            additionalProperties: boolean;
            required: string[];
        };
        strings: {
            type: string;
            patternProperties: {
                ".": {
                    type: string;
                };
            };
            additionalProperties: boolean;
        };
        files: {
            type: string;
            items: {
                $ref: string;
            };
        };
        file: {
            type: string;
            properties: {
                name: {
                    type: string;
                };
                description: {
                    type: string;
                };
                copyLocation: {
                    type: string;
                };
            };
            additionalProperties: boolean;
            required: string[];
        };
        keyboards: {
            type: string;
            items: {
                $ref: string;
            };
        };
        keyboard: {
            type: string;
            properties: {
                name: {
                    type: string;
                };
                id: {
                    type: string;
                };
                version: {
                    type: string;
                };
                oskFont: {
                    type: string;
                };
                displayFont: {
                    type: string;
                };
                rtl: {
                    type: string;
                };
                languages: {
                    type: string;
                    items: {
                        $ref: string;
                    };
                };
                examples: {
                    type: string;
                    items: {
                        $ref: string;
                    };
                };
                webOskFonts: {
                    type: string;
                    items: {
                        type: string;
                    };
                };
                webDisplayFonts: {
                    type: string;
                    items: {
                        type: string;
                    };
                };
            };
            additionalProperties: boolean;
            required: string[];
        };
        keyboardLanguage: {
            type: string;
            properties: {
                name: {
                    type: string;
                };
                id: {
                    type: string;
                };
            };
            additionalProperties: boolean;
            required: string[];
        };
        info: {
            type: string;
            properties: {
                website: {
                    $ref: string;
                };
                version: {
                    $ref: string;
                };
                name: {
                    $ref: string;
                };
                copyright: {
                    $ref: string;
                };
                author: {
                    $ref: string;
                };
                description: {
                    $ref: string;
                };
            };
            additionalProperties: boolean;
        };
        infoItem: {
            type: string;
            properties: {
                description: {
                    type: string;
                };
                url: {
                    type: string;
                };
            };
            additionalProperties: boolean;
            required: string[];
        };
        lexicalModels: {
            type: string;
            items: {
                $ref: string;
            };
        };
        lexicalModel: {
            type: string;
            properties: {
                name: {
                    type: string;
                };
                id: {
                    type: string;
                };
                version: {
                    type: string;
                };
                rtl: {
                    type: string;
                };
                languages: {
                    type: string;
                    items: {
                        $ref: string;
                    };
                };
            };
            additionalProperties: boolean;
            required: string[];
        };
        keyboardExample: {
            type: string;
            properties: {
                id: {
                    type: string;
                };
                keys: {
                    type: string;
                };
                text: {
                    type: string;
                };
                note: {
                    type: string;
                };
            };
            additionalProperties: boolean;
            required: string[];
        };
        relatedPackage: {
            type: string;
            properties: {
                id: {
                    type: string;
                };
                relationship: {
                    type: string;
                    enum: string[];
                };
            };
            additionalProperties: boolean;
            required: string[];
        };
    };
};
export default _default;
//# sourceMappingURL=kmp.schema.d.ts.map