import { constants } from "@keymanapp/ldml-keyboard-constants";
import { KMXPlus } from "@keymanapp/common-types";
import { build_strs_index } from "./build-strs.js";
var ListItem = KMXPlus.ListItem;
;
;
;
export function build_list(source_list, sect_strs) {
    if (!source_list?.lists?.length) {
        // there's always the null list
        return null;
    }
    const result = {
        ident: constants.hex_section_id(constants.section.list),
        size: 0,
        _offset: 0,
        listCount: source_list.lists.length,
        indexCount: 0,
        lists: [],
        indices: [],
    };
    result.lists = source_list.lists.map(array => {
        const list = {
            index: result.indices.length, // the next indexcount
            count: array.length,
            _value: array
        };
        array.forEach((i) => {
            const index = {
                // Get the final string index
                str: build_strs_index(sect_strs, i.value),
                _value: i.value.value, // unwrap the actual string value
            };
            result.indices.push(index); // increment the indexCount
            result.indexCount++;
        });
        return list;
    });
    // Sort the lists.
    result.lists.sort((a, b) => a._value.compareTo(b._value));
    const offset = constants.length_list +
        (constants.length_list_item * result.listCount) +
        (constants.length_list_index * result.indexCount);
    result.size = offset;
    return result;
}
/**
 * Returns the index into the list, analagous to build_strs_index
 * @param sect_strs
 * @param value
 * @returns
 */
export function build_list_index(sect_list, value) {
    if (!value) {
        return 0; // empty list
    }
    if (!(value instanceof ListItem)) {
        throw new Error('unexpected value ' + value);
    }
    const result = sect_list.lists.findIndex(v => v._value === value);
    if (result < 0) {
        throw new Error('unexpectedly missing ListItem ' + value); // TODO-LDML: it's an array of strs
    }
    return result;
}
//# sourceMappingURL=build-list.js.map