import { constants } from "@keymanapp/ldml-keyboard-constants";
import { KMXPlus } from "@keymanapp/common-types";
import { build_strs_index } from "./build-strs.js";
var UsetItem = KMXPlus.UsetItem;
;
;
/**
* Builder for the 'uset' section
*/
export function build_uset(kmxplus, sect_strs) {
    if (!kmxplus?.uset?.usets) {
        return null;
    }
    // uset items in pattern order
    const items = [...kmxplus.uset.usets]
        .sort((a, b) => a.compareTo(b));
    const ranges = [];
    const usets = [];
    items.forEach(set => {
        usets.push({
            range: ranges.length, // index
            count: set.uset.length,
            pattern: build_strs_index(sect_strs, set.str),
            _pattern: set.str,
        });
        // append the ranges
        set.uset.ranges.forEach(([start, end]) => ranges.push({ start, end }));
    });
    const uset = {
        ident: constants.hex_section_id(constants.section.uset),
        size: constants.length_uset +
            (constants.length_uset_uset * usets.length) +
            (constants.length_uset_range * ranges.length),
        usetCount: usets.length,
        rangeCount: ranges.length,
        usets,
        ranges,
        _offset: 0,
    };
    return uset;
}
/**
 * @returns uset index
 */
export function build_uset_index(sect_uset, value) {
    if (!(value instanceof UsetItem)) {
        if (value === null) {
            throw new Error('unexpected null UsetItem');
        }
        else {
            throw new Error('Expected UsetItem but got ' + value);
        }
    }
    const result = sect_uset.usets.findIndex(v => v._pattern.value === value.str.value);
    if (result < 0) {
        throw new Error('unexpectedly missing UsetItem ' + value.uset.toString());
    }
    return result;
}
//# sourceMappingURL=build-uset.js.map