import { KeymanFileTypes } from '@keymanapp/common-types';
import { CompilerAsyncCallbacks, CompilerPathCallbacks } from "../../compiler-callbacks.js";
export declare class KeymanDeveloperProject {
    private _projectFilename;
    protected callbacks: CompilerAsyncCallbacks;
    options: KeymanDeveloperProjectOptions;
    files: KeymanDeveloperProjectFile[];
    projectPath: string;
    readonly projectFile: KeymanDeveloperProjectFile;
    get projectFilename(): string;
    constructor(_projectFilename: string, version: KeymanDeveloperProjectVersion, callbacks: CompilerAsyncCallbacks);
    /**
     * Adds .kmn, .xml, .kps to project based on options.sourcePath
     * @param projectFilename Full path to project.kpj (even if the file doesn't exist)
     */
    populateFiles(): Promise<boolean>;
    isKeyboardProject(): boolean;
    isLexicalModelProject(): boolean;
    private resolveProjectPath;
    resolveBuildPath(): string;
    getOutputFilePath(type: KeymanFileTypes.Binary): string;
    resolveInputFilePath(file: KeymanDeveloperProjectFile): string;
    resolveOutputFilePath(file: KeymanDeveloperProjectFile, sourceExt: string, targetExt: string): string;
}
export declare enum KeymanDeveloperProjectType {
    Keyboard = 0,
    LexicalModel = 1
}
export type KeymanDeveloperProjectVersion = "1.0" | // Keyman Developer <17.0: All files referenced in .kpj
"2.0";
export declare class KeymanDeveloperProjectOptions {
    buildPath: string;
    sourcePath: string;
    compilerWarningsAsErrors: boolean;
    warnDeprecatedCode: boolean;
    checkFilenameConventions: boolean;
    /**
     * Skip building .keyboard_info and .model_info files, for example in
     * unit tests or for legacy keyboards
     */
    skipMetadataFiles: boolean;
    projectType: KeymanDeveloperProjectType;
    readonly version: KeymanDeveloperProjectVersion;
    constructor(version: KeymanDeveloperProjectVersion);
}
export interface KeymanDeveloperProjectFile {
    get filename(): string;
    get fileType(): string;
    readonly filePath: string;
}
export declare class KeymanDeveloperProjectFile10 implements KeymanDeveloperProjectFile {
    readonly id: string;
    readonly filePath: string;
    readonly fileVersion: string;
    private path;
    get filename(): string;
    get fileType(): string;
    details: KeymanDeveloperProjectFileDetail_Kmn & KeymanDeveloperProjectFileDetail_Kps;
    childFiles: KeymanDeveloperProjectFile[];
    constructor(id: string, filePath: string, fileVersion: string, path: CompilerPathCallbacks);
}
export type KeymanDeveloperProjectFileType20 = KeymanFileTypes.Source;
export declare class KeymanDeveloperProjectFile20 implements KeymanDeveloperProjectFile {
    readonly filePath: string;
    private path;
    get filename(): string;
    get fileType(): string;
    constructor(filePath: string, path: CompilerPathCallbacks);
}
export declare class KeymanDeveloperProjectFileDetail_Kps {
    name?: string;
    copyright?: string;
    version?: string;
}
export declare class KeymanDeveloperProjectFileDetail_Kmn {
    name?: string;
    copyright?: string;
    message?: string;
}
//# sourceMappingURL=keyman-developer-project.d.ts.map