import { VisualKeyboard as VK, Constants } from '@keymanapp/common-types';
import { KeymanXMLWriter } from '../../index.js';
var USVirtualKeyCodes = Constants.USVirtualKeyCodes;
var VisualKeyboardLegalShiftStates = VK.VisualKeyboardLegalShiftStates;
export default class KVKSFileWriter {
    write(vk) {
        const flags = {};
        if (vk.header.flags & 2 /* VisualKeyboardHeaderFlags.kvkhDisplayUnderlying */) {
            flags.displayunderlying = '';
        }
        if (vk.header.flags & 1 /* VisualKeyboardHeaderFlags.kvkh102 */) {
            flags.key102 = '';
        }
        if (vk.header.flags & 8 /* VisualKeyboardHeaderFlags.kvkhAltGr */) {
            flags.usealtgr = '';
        }
        if (vk.header.flags & 4 /* VisualKeyboardHeaderFlags.kvkhUseUnderlying */) {
            flags.useunderlying = '';
        }
        const kvks = {
            visualkeyboard: {
                header: {
                    version: '10.0',
                    kbdname: vk.header.associatedKeyboard,
                    flags: flags,
                },
                encoding: []
            }
        };
        if (vk.header.underlyingLayout)
            kvks.visualkeyboard.header.layout = vk.header.underlyingLayout;
        const encodings = { ansi: null, unicode: null };
        for (const key of vk.keys) {
            const encoding = key.flags & 2 /* VisualKeyboardKeyFlags.kvkkUnicode */ ? 'unicode' : 'ansi';
            const shift = this.kvkShiftToKvksShift(key.shift);
            if (!encodings[encoding]) {
                encodings[encoding] = {
                    o: {
                        layer: [],
                        $: { name: encoding,
                            fontname: encoding == 'ansi' ? vk.header.ansiFont.name : vk.header.unicodeFont.name,
                            fontsize: (encoding == 'ansi' ? vk.header.ansiFont.size : vk.header.unicodeFont.size).toString(),
                        },
                    },
                    l: {}
                };
                kvks.visualkeyboard.encoding.push(encodings[encoding].o);
            }
            const e = encodings[encoding];
            if (!e.l[shift]) {
                e.l[shift] = {
                    key: [],
                    $: { shift: shift },
                };
                e.o.layer.push(e.l[shift]);
            }
            const l = e.l[shift];
            // TODO-LDML: map
            let vkeyName = '';
            for (const vkey of Object.keys(USVirtualKeyCodes)) {
                if (USVirtualKeyCodes[vkey] == key.vkey) {
                    vkeyName = vkey;
                    break;
                }
            }
            if (vkeyName == '') {
                //TODO-LDML: warn
                continue;
            }
            const k = {
                $: { vkey: vkeyName },
                _: key.text,
            };
            if (key.bitmap) {
                k.bitmap = this.arrayToBase64(key.bitmap);
            }
            l.key.push(k);
        }
        const result = new KeymanXMLWriter('kvks').write(kvks);
        return result; //Uint8Array.from(result);
    }
    arrayToBase64(source) {
        const ascii = String.fromCharCode(...source);
        return btoa(ascii);
    }
    kvkShiftToKvksShift(shift) {
        // TODO-LDML(lowpri): make a map of this?
        for (const state of VisualKeyboardLegalShiftStates) {
            if (state.shift == shift) {
                return state.name;
            }
        }
        return '';
    }
}
//# sourceMappingURL=kvks-file-writer.js.map